﻿using System;
using System.Linq;
using System.ComponentModel.DataAnnotations;

using IdeaBlade.EntityModel;
using IdeaBlade.Core;
using IdeaBlade.Validation;

namespace DomainModel {

  // Customer class with sample property interceptors.
  public partial class Customer  {

    // Force the Country to upper case for display purposes.
    [AfterGet(EntityPropertyNames.Country)]
    public String UppercaseCountry(String country) {
      if (!String.IsNullOrEmpty(country)) {
        return country.ToUpper();
      } else {
        return country;
      }
    }

    // Log the set of any customer property.
    [AfterSet]
    public void AfterSetAnyProperty(IEntityPropertyInterceptorArgs args) {
      string msg = string.Format("Setting {0} to {1}", args.EntityProperty.Name,
        args.Value == null ? "null" : args.Value.ToString());
      TraceLogger(msg);
    }

    // Force the CompanyName to upper case when set.
    [BeforeSet("CompanyName", Key = "C1")]
    public void BeforeSetCompanyName(DataEntityPropertySetInterceptorArgs<Customer, string> args) {
      if (args.Value != null) {
        args.Value = args.Value.ToUpper();
      }
    }

    // Log when CompanyName is set.
    [AfterSet("CompanyName", Key = "C2")]
    public void AfterSetCompanyName(DataEntityPropertySetInterceptorArgs<Customer, string> args) {
      string msg = string.Format("Set {0} to {1}", args.DataEntityProperty.Name,
        args.Value == null ? "null" : args.Value);
      TraceLogger(msg);
    }


    private static void TraceLogger(string msg) {
      System.Diagnostics.Debug.WriteLine(msg);
    }

  }
  
}
