﻿#Region "usings"
Imports System.Collections
Imports System.Windows
Imports System.Windows.Controls

Imports IdeaBlade.EntityModel
Imports IdeaBlade.EntityModel.Extensions
Imports IdeaBlade.Core

Imports DomainModel

#End Region

Namespace SilverlightPIApp

  Partial Public Class Page
	  Inherits UserControl

	#Region "ctor & load"

	Public Sub New()
	  InitializeComponent()
	  AddHandler Loaded, AddressOf Page_Loaded
	End Sub

	Private Sub Page_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  InitializePersistence()
	  InitializeQueries()
	End Sub

	Private Sub InitializePersistence()
	  DynamicInterceptors.AddDynamicEntityInterceptors()
	  _entityManager = New NorthwindIBEntities(True)
	  Login()
	End Sub

	#End Region

	#Region "Button Click Handlers"

	Private Sub btnSave_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  Save()
	End Sub

	#End Region

	#Region "Initializequeries"

	Private Sub InitializeQueries()
	  AddQueries()
	  InitializeQueryCombo()
	End Sub

	Private Sub InitializeQueryCombo()

      _queryCombo.ItemsSource = _queries.Keys.ToList()
      _queryCombo.SelectedItem = _queries.Keys.FirstOrDefault()
      AddHandler _queryCombo.SelectionChanged, Sub() Fetch()
	End Sub

	#End Region

	#Region "Fetch"

	Private Sub Fetch()

	  Dim query = GetQuery()
	  If Nothing Is query Then
		WriteMessage("No query selected; pick a query.")
		Return
	  End If

	  WriteMessage("Fetching ...")

	  _entityManager.ExecuteQueryAsync(query, AddressOf FetchCallback, Nothing)
	End Sub

	Private Sub FetchCallback(ByVal args As EntityQueryOperation)
	  If args.HasError Then
		WriteMessage(args.Error.Message)
		Return
	  End If

	  ' Special logic to handle the projection query - 
	  '  can't bind to an anonymous type in Silverlight
	  If AnonymousFns.IsAnonymousType(args.EntityQuery.ElementType) Then
		_dataGrid.ItemsSource = DynamicTypeConverter.Convert(args.Results)
	  Else
		_dataGrid.ItemsSource = args.Results
	  End If

	  ReportFetchCount(args.Results)
	End Sub

	Private Function GetQuery() As IEntityQuery
	  Dim queryName = CStr(_queryCombo.SelectedItem)
	  Return If(queryName Is Nothing, Nothing, _queries(queryName))
	End Function

	Private Sub ReportFetchCount(ByVal result As IEnumerable)
	  WriteMessage(String.Format("Retrieved {0} customers", result.Cast(Of Object)().Count()))
	End Sub

	#End Region

	#Region "Queries "

	Private Sub AddQueries()

	  _queries = New Dictionary(Of String, IEntityQuery)()

	  _queries.Add("Get all Customers", _entityManager.Customers)

	  _queries.Add("Get Customers starting with A", _entityManager.Customers.Where(Function(c) c.CompanyName.StartsWith("A")))

	  _queries.Add("Get Customers with Orders in Oregon", _entityManager.Customers.Where(Function(c) c.Orders.Any(Function(o) o.ShipRegion = "OR")))

	  _queries.Add("Get Customers starting with B from cache-only", _entityManager.Customers.Where(Function(c) c.CompanyName.StartsWith("B")).With(QueryStrategy.CacheOnly))

	  _queries.Add("Get Customer info as projection", _entityManager.Customers.Select(Function(c) New With {Key c.CustomerID, Key c.CompanyName, Key c.City}))
	End Sub

	#End Region

	#Region "Login"

	Private Sub Login()
	  Dim cred = New LoginCredential("demo", "demo", "demo")

	  WriteMessage("Logging in ...")

	  _entityManager.LoginAsync(cred, AddressOf LoginCallback, Nothing)
	End Sub

	Private Sub LoginCallback(ByVal args As LoginOperation)
	  If args.HasError Then
		WriteMessage(args.Error.Message)
		Return
	  End If

	  WriteMessage("Logged in")

	  _queryCombo.IsEnabled = args.IsCompleted
	  _btnSave.IsEnabled = args.IsCompleted
	  Fetch()
	End Sub

	#End Region

	#Region "Save"

	Private Sub Save()

	  If Not _entityManager.HasChanges() Then
		WriteMessage("No changes to save.")
		Return
	  End If

	  WriteMessage("Saving ...")

	  _entityManager.SaveChangesAsync(AddressOf SaveCallback, Nothing)
	End Sub

	Private Sub SaveCallback(ByVal args As EntitySaveOperation)
	  If args.IsCompleted Then
		WriteMessage("Changes saved")
	  Else
		WriteMessage(args.Error.Message)
	  End If
	End Sub

	#End Region

	#Region "Misc"

	Private Sub WriteMessage(ByVal msg As String)
	  _txtStatus.Text = msg
	End Sub

	#End Region

	#Region "Fields"

	Private _entityManager As NorthwindIBEntities
	Private _queries As Dictionary(Of String, IEntityQuery)

	#End Region
  End Class
End Namespace
